<?php


/**
 * Automatically create a person
 *
 * @param string $name The person's name
 * @return int
 */
function moviedb_person_add($name) {
  global $user;
  $actnode->uid = $user->uid;
  $actnode->name = $user->name;
  $actnode->type = 'mdb_person';
  $actnode->title = $name;
  $actnode->status = 1;

  //Make sure the data is ok and save him
  node_validate($actnode);
  if (!form_get_errors()) {
    $actnode = node_submit($actnode);
    node_save($actnode);
    return $actnode->nid;
  }
  else {
    return FALSE;
  }
}

/**
 * Automatically create a movie
 *
 * @param string $name The title of the movie
 * @param int $year The release year of the movie
 * @return int
 */
function moviedb_movie_add($name, $year) {
  global $user;
  $actnode->uid = $user->uid;
  $actnode->name = $user->name;
  $actnode->type = 'mdb_movie';
  $actnode->title = $name;
  $actnode->releaseyear = $year;
  $actnode->status = 0;

  //Make sure the data is ok and save him
  node_validate($actnode);
  if (!form_get_errors()) {
    $actnode = node_submit($actnode);
    node_save($actnode);
    return $actnode->nid;
  }
  else {
    return FALSE;
  }
}

/**
 * Check if a person with that name exists in the databasse
 *
 * @param string $name
 * @return int
 */
function moviedb_person_exists($name) {

  $result = db_query_range("SELECT nid FROM {node} WHERE title = '%s' AND type = '%s'", $name, 'mdb_person', 0, 1);

  if ($person = db_fetch_object($result)) {
    return $person->nid;
  }
  else {
    return FALSE;
  }

}

/**
 * Check if a movie  with that title and released in that year exists in
 *  the database
 *
 * @param string $name
 * @param int $year
 * @return int
 */
function moviedb_movie_exists($name, $year) {

  $result = db_query_range("SELECT moviedb.nid FROM {moviedb} INNER JOIN {node} ON node.nid = moviedb.nid WHERE node.title  = '%s' AND moviedb.releaseyear = '%s'", $name, $year, 0, 1);

  if ($movie = db_fetch_object($result)) {
    return $movie->nid;
  }
  else {
    return FALSE;
  }

}


/**   
 * Get the id of a movie.
 *
 * @param string $name
 * @param int $year
 * @return nid
 */
function moviedb_movie_get_id($name, $year) {
  
  $id = moviedb_movie_exists($name, $year);
  if (!$id) {
    $id = moviedb_movie_add($name, $year);
  }

  return $id;
} 
      

/**
 * Get the id of a person.
 *
 * @param string $name
 * @return nid
 */
function moviedb_person_get_id($name) {
  $id = moviedb_person_exists($name);
  if (!$id) {
    $id = moviedb_person_add($name);
  }

  return $id;
}

/**
 * Process people sent by the form
 *
 * @param int $mid the movie id
 * @param string $string the raw submitted people string
 * @param obj $mdb_job The job object 
 */
function moviedb_people_process($mid, $string, $mdb_job) {

  if (!$mdb_job->table_name) {
    return FALSE;
  }

  //Put the people in a table
  $people = explode(',', $string);

  //Loop through the people
  foreach ($people as $weight => $person) {

    //Trim the name
    $person = trim($person);
    
    list($name, $data) = explode("::", $person);

    $name = trim($name);
    $data = trim($data);

    //If the name is empty, go to next
    if (!$name) {
      $data = "";
      continue;
    }

    //Get the id..
    $person_id = moviedb_person_get_id($name);

    //Something's wrong. don't add him
    if (!$person_id) {
      continue;
    }

    //Add the person to the movie
    moviedb_movie_person_join($mdb_job, $mid, $person_id, $weight, $data);
    $data = "";
  }
}

/**
 * Function for adding movies to a person in a specific job
 *
 * @param int $pid Person's node ID
 * @param string $movies Specifically formatted list of movies person was involved in.
 * @param obj $mdb_job The job object
 */
function moviedb_movies_process($pid, $movies, $mdb_job) {
  
  if (!$mdb_job->table_name) {
    return FALSE;
  }
  //Put the people in a table
  $movies = explode("\n", $movies);

  //Loop through the people
  foreach ($movies as $movie) {

    //Trim the name
    $movie = trim($movie);

    //Get that year out of the string
    $movie = _moviedb_movies_process($movie);

    if (!is_numeric($movie["weight"])) {

      $movie["weight"] = 9;

    }

    //If the name is empty, go to next
    if (!$movie["title"] || !$movie["releaseyear"]) {
      continue;
    }

    //Get the id...
    $movie_id = moviedb_movie_get_id($movie["title"], $movie["releaseyear"]);

    //Something's wrong. don't add the movie
    if (!$movie_id) {
      continue;
    }
    moviedb_movie_person_join($mdb_job, $movie_id, $pid, $movie["weight"], $movie["data"]);
    
  }
}

/**
 * Helper function for moviedb_movies_process
 *
 * @param string $string The input from the mdb_person add/edit form
 * @return array
 */
function _moviedb_movies_process($string) {

  $pattern = "/^(.*) \(([0-9]{4})\)(?: \[([0-9]*)\])?(?: ::(.*))?$/";

  preg_match($pattern, $string, $matches);
  $matches = array_pad($matches, 5, FALSE);
  $keys = array("string", "title", "releaseyear", "weight", "data");
  
  $movie = array_combine($keys, $matches); 

  return array_map("trim", $movie);

}

/**
 * Link a movie and a person together with a specific job type in the database
 * 
 * @param obj $mdb_job The job object
 * @param int $mid The movie's node ID
 * @param int $pid The person's node ID
 * @param int $weight The person's weighted importance in their job for the movie
 * @param string $data Any miscellaneous information (character names, etc) related to this relationship
 */
function moviedb_movie_person_join($mdb_job, $mid, $pid, $weight, $data) {

  if (!$mdb_job->table_name) {
    return FALSE;
  }  
  elseif ($mdb_job->table_name == 'moviedb_misc') {
    db_query("INSERT INTO {moviedb_misc} (mid, pid, jid, weight, data) VALUES (%d,%d, %d, %d, '%s')", $mid, $pid, $mdb_job->jid, $weight, check_plain($data));
  } 
  else {
    $table = $mdb_job->table_name;

    db_query("INSERT INTO {$table} (mid, pid, weight, data) VALUES (%d, %d, %d, '%s')", $mid, $pid, $weight, check_plain($data));
   }
}


/**
 * Retrieve people associated with a movie in a particular job from the database
 *
 * @param int $mid The movie id
 * @param obj $mdb_job The job object
 * @return array
 */
function moviedb_people_load($mid, $mdb_job) {

  if (!$mdb_job->table_name) {
    return FALSE;
  } 
  elseif ($mdb_job->table_name == 'moviedb_misc') {
    $people = db_query("SELECT p.title, p.nid, j.weight, j.data FROM {node} as p INNER JOIN {moviedb_misc} as j ON j.pid = p.nid WHERE j.mid = %d AND j.jid = %d ORDER BY j.weight ASC", $mid, $mdb_job->jid); 
  } 
  else { 
    $people = db_query("SELECT p.title, p.nid, j.weight, j.data FROM {node} as p INNER JOIN {%s} as j ON j.pid = p.nid WHERE j.mid = %d ORDER BY j.weight ASC", $mdb_job->table_name, $mid);
  }
  $return = array("object" => array());
  $formdata = "";

  while ($person = db_fetch_object($people)) {
    $return['object'][] = $person;

    if ($person->data) {
      $formdata = ' ::'. $person->data;
    }

    if (@$return['form']) {
      $return['form'] .= ', '. $person->title . $formdata;
    }
    else {
      $return['form'] = $person->title . $formdata;
    }
    $formdata = "";
  }

  return $return;
}

/**
 * Retrieve movies associated with a specific person and role from the database
 *
 * @param int $mid The movie id
 * @param obj $mdb_job The job object
 * @return array
 */
function moviedb_person_movies($pid, $mdb_job) {

  if (!$mdb_job->table_name) {
    return FALSE;
  } 
  elseif ($mdb_job->table_name == "moviedb_misc") {
    $movies = db_query("SELECT p.*, m.*, j.* FROM {node} AS p INNER JOIN {%s} AS j ON j.mid = p.nid INNER JOIN {%s} AS m ON m.nid = p.nid WHERE j.pid = %d AND j.jid = %d ORDER BY m.releaseyear ASC, p.title ASC", $mdb_job->table_name, "moviedb", $pid, $mdb_job->jid);
  } 
  else {
    $movies = db_query("SELECT p.*, m.*, j.* FROM {node} as p INNER JOIN {%s} as j ON j.mid = p.nid INNER JOIN {%s} as m ON m.nid = p.nid WHERE j.pid = %d ORDER BY m.releaseyear ASC, p.title ASC", $mdb_job->table_name, "moviedb", $pid);
  }
  $return = FALSE;

  while ($movie = db_fetch_object($movies)) {
    $return['object'][] = $movie;
    $formdata="";

    if ($movie->releaseyear) {
      $formdata .= ' ('. $movie->releaseyear .')';
    }
    if (is_numeric($movie->weight)) {
      $formdata .= ' ['. $movie->weight .']';
    } 
    else {
      $formdata .= ' [9]';
    }

    if ($movie->data) {
      $formdata .= ' ::'. $movie->data;
    }

    if ($return['form']) {
        $return['form'] .= "\n". $movie->title . $formdata;
    }
    else {
        $return['form'] .= $movie->title . $formdata;
    }
    $formdata = "";
  }

  return $return;
}

/**
 * Utility function to sort the filmography
 *
 * @param array $a
 * @param array $b
 * @return int
 */
function moviedb_person_usort($a, $b) {
  return count($b) - count($a);
}

/**
 * Randomly useful function to return number of movies in your database.
 * Accepts UID so that you can get a number of movies authored by a specific user
 *
 * @param int $uid
 * @return int
 */
 
function mdb_movie_review_count($uid="") {
  if (!empty($uid)) {
    $userwhere = " AND node.uid = $uid ";
  }
   
  $sql = "SELECT count(*) AS cnt FROM {node} WHERE node.type='mdb_movie' AND node.promote = 1 AND node.status = 1 ". $userwhere;

  $result = db_fetch_object(db_query($sql));
  
  $cnt = $result->cnt;

  return $cnt;
}

function moviedb_count_rows() {
  return;
}

