<?php

/**
 * Implementation of hook_access().
 */
function mdb_person_access($op, $node) {
  global $user;

  if ($op == 'create') {
    return user_access('create mdb_movie');
  }

  if ($op == 'update' || $op == 'delete') {
    if (user_access('edit mdb_person')) {
      return TRUE;
    }
  }
}

/**
 * Implementation of hook_insert()
 */
function mdb_person_insert($node) {

  $jobs = moviedb_get_jobs();

  foreach ($jobs as $mdb_job) { 
    $role_form = $mdb_job->job ."_form";
    moviedb_movies_process($node->nid, $node->$role_form, $mdb_job);
  }
}


/**
 * Implementation of hook_delete()
 *
 */
function mdb_person_delete($node) {

  $jobs = moviedb_get_jobs();

  foreach ($jobs as $mdb_job) {
    db_query("DELETE FROM {%s} WHERE pid = %d", $mdb_job->table_name, $node->nid);
  }
}


/**
 * Implementation of hook_update()
 *
 */
function mdb_person_update($node) {

  $jobs = moviedb_get_jobs();

  foreach ($jobs as $mdb_job) {
    $role_table = $mdb_job->table_name; 
    $role_form = $mdb_job->job ."_form";
    //Delete all the records from the actor links table
    if ($role_table == "moviedb_misc") {
      db_query("DELETE FROM {%s} WHERE pid = %d AND jid = %d", $role_table, $node->nid, $mdb_job->jid);
    } 
    else { 
      db_query("DELETE FROM {%s} WHERE pid = %d", $role_table, $node->nid);
    }
    //Add the actors
    moviedb_movies_process($node->nid, $node->$role_form, $mdb_job);
  }
}
/**
 * Implementation of hook_load().
 *
 */
function mdb_person_load($node) {

 //Get the person's filmography

  $jobs = moviedb_get_jobs();

  foreach ($jobs as $mdb_job) { 

    $role       = $mdb_job->job;
    $role_form  = $role ."_form";
    
    $role_load          = moviedb_person_movies($node->nid, $mdb_job);
    $movies->$role      = $role_load["object"];
    $movies->$role_form = $role_load["form"];

  }

  return $movies;

}

function mdb_person_form(&$node) {

  $form['title'] = array(    
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $node->title,
    '#size' => 60,
    '#maxlength' => 255,
    '#description' => t('The name of the person.'),
  );

  $form['body'] = array(   
    '#type' => 'textarea',    
    '#title' => t('Biography'),    
    '#default_value' => $node->body,    
    '#cols' => 50,    
    '#rows' => 10,    
    '#description' => t('A biography of the person in question'),  
  );  
  
  $form['movies'] = array(    
    '#type'         => 'fieldset',    
    '#title'        => t('Movies'),    
    '#collapsible'  => TRUE,    
    '#weight'       => 2,  
  );  
  $jobs = moviedb_get_jobs();

  foreach ($jobs as $mdb_job) {
    
    $role       = $mdb_job->job;
    $role_form  = $role ."_form";
    $role_title = ucfirst($mdb_job->plural);

    $form['movies'][$role_form] = array(   
      '#type'           => 'textarea',    
      '#title'          => t($role_title),
      '#default_value'  => $node->$role_form,
      '#description'    => t('Enter the name of the films where person performed as '. $role .' in Title (Releaseyear) format, one film per line. Example: Brazil (1985).'),
      '#cols'           => 50,
      '#rows'           =>  4,
      '#weight'         =>  $mdb_job->weight,
    );
  }
  $form['format'] = filter_form($node->format, 8);

  return $form;
}

function mdb_person_autocomplete($string) {

  // The user enters a comma-separated list of tags. We only autocomplete the last tag.
  // This regexp allows the following types of user input:
  // this, "somecmpany, llc", "and ""this"" w,o.rks", foo bar
  $regexp = '%(?:^|,\ *)("(?>[^"]*)(?>""[^"]* )*"|(?: [^",]*))%x';
  preg_match_all($regexp, $string, $matches);
  $array = $matches[1];

  // Fetch last tag
  $last_string = strtolower(trim(array_pop($array)));
  if ($last_string != '') {
    $result = db_query_range(db_rewrite_sql("SELECT title FROM {node} WHERE LOWER(title) LIKE '%s%%' AND type='%s'"), $last_string, "mdb_person", 0, 10);

    $prefix = count($array) ? implode(', ', $array) .', ' : '';

    $matches = array();
    while ($person = db_fetch_object($result)) {
      $n = $person->title;
      // Commas and quotes in terms are special cases, so encode 'em.
      if (preg_match('/,/', $person->title) || preg_match('/"/', $person->title)) {
        $n = '"'. preg_replace('/"/', '""', $person->title) .'"';
      }
      $matches[$prefix . $n] = check_plain($person->title);
    }
    print drupal_to_js($matches);
    exit();
  }
}
