<?php

function moviedb_theme() {
  return array(
    'mdb_movie' => array(
      'template' => 'mdb_movie',
      'arguments' => array('node' => NULL, 'teaser' => FALSE, 'page' => FALSE)
    ),
    'mdb_person' => array(
      'template' => 'mdb_person',
      'arguments' => array('node' => NULL, 'teaser' => FALSE, 'page' => FALSE)
    ),
    'mdb_person_filmography' => array(
      'arguments' => array('movies' => NULL, 'mdb_job' => NULL)
    ),
    'mdb_movie_castcrew' => array(
      'arguments' => array('people' => NULL, 'mdb_job' => NULL)
    ),
    'mdb_movie_title' => array(
      'arguments' => array('title' => NULL)
    )
  );
}

/**
 * Implementation of hook_view().
 */
function mdb_movie_view(&$node, $teaser = FALSE, $page = FALSE) {
  if ($page) {
    drupal_set_breadcrumb(array(l(t('Home'), ''), l(t('Movies'), 'movie')));
    drupal_add_css(drupal_get_path('module', 'moviedb') .'/moviedb.css');
  }
  $node = node_prepare($node, $teaser);

  $node->content['body'] = array(
    '#value' => $teaser ? $node->teaser : theme('mdb_movie', $node, $teaser, $page),
    '#weight' => 1,
  );
  return $node;
}

function mdb_person_view(&$node, $teaser = FALSE, $page = FALSE) {

  if ($page) {
    drupal_set_breadcrumb(array(l(t('Home'), ''), l(t('People'), 'people')));
    drupal_add_css(drupal_get_path('module', 'moviedb') .'/moviedb.css');
  }
             
  
  $node = node_prepare($node, $teaser);

  $node->content['body'] = array(
    '#value' => $teaser ? $node->teaser : theme('mdb_person', $node, $teaser, $page),
    '#weight' => 1,
  );
  return $node;
}

function template_preprocess_mdb_movie(&$variables) {
   $jobs = moviedb_get_jobs();

   $new_title = $variables["node"]->title ." (". $variables["node"]->releaseyear .")";

   drupal_set_title($new_title);

   foreach ($jobs as $mdb_job) {
     $role = $mdb_job->job;
     if ($variables["node"]->$role) {
       $variables["castcrew"][$mdb_job->weight] = theme("mdb_movie_castcrew", $variables["node"]->$role, $mdb_job, 0);    
     }
   }
   ksort($variables["castcrew"]);
   $variables["jobs"] = $jobs;
   template_preprocess_node($variables);

}

function template_preprocess_mdb_person(&$variables) {
  $jobs = moviedb_get_jobs();

  foreach ($jobs as $mdb_job) {
    $role = $mdb_job->job;
    if ($variables["node"]->$role) {
      $roles[$mdb_job->job] = $variables["node"]->$role;
      $jobhold[$mdb_job->job] = $mdb_job;
    }
  }
  uasort($roles, 'moviedb_person_usort');
  foreach ($roles as $job => $role) {
    $variables["filmography"][] = theme("mdb_person_filmography", $role, $jobhold[$job]);
  }
}
function theme_mdb_person_filmography($movies, $mdb_job) {

  $rows = array();

  foreach ($movies as $movie) {
    $rows[] = array(array("data" => $movie->releaseyear, "class" => "field-releaseyear"), array("data" => l($movie->title, "node/". $movie->nid)), array("data" => ($movie->data)));
  }

  $header = array(array("data" => "Movies as ". ucfirst($mdb_job->job), "colspan" => 2));

  return theme("table", $header, $rows);

}

function theme_mdb_movie_castcrew($people, $mdb_job) {
  
   $limit = (is_numeric($mdb_job->display_limit)) ? $mdb_job->display_limit : 0;

   $items = array();
   foreach ($people as $person) {
     if ($person->data) { 
       $person->data = " (". $person->data .")";
     }
     $items[] = l($person->title, "node/". $person->nid) . $person->data;
     if ($limit > 0 && count($items) >= $limit) {
       break;
     }
   }
   return theme("item_list", $items, ucwords($mdb_job->plural) .": ");
}   
   

/** 
 * Render movie titles with the "The" in front
 */
function theme_mdb_movie_title($title) {
  $title = trim($title);
    
  if (substr($title,-5) == ", The") {
    $title = "The ".substr($title,0,-5);  
  } 
  elseif (substr($title,-4) == ", An") {
    $title = "An ".substr($title,0,-4);
  } 
  elseif (substr($title,-3) == ", A") {    
    $title = "A ".substr($title,0,-3);
  }   
  return $title;
}    